Package: LACE
Version: 2.16.0
Date: 2026-03-09
Title: Longitudinal Analysis of Cancer Evolution (LACE)
Authors@R: c(person("Daniele", "Ramazzotti", role=c("aut"),email="daniele.ramazzotti@unimib.it",
                comment = c(ORCID = "0000-0002-6087-2666")),
    person("Fabrizio","Angaroni",role=c("aut"),email="fabrizio.angaroni@unimib.it"),
    person("Davide","Maspero",role=c("cre","aut"),email="d.maspero@campus.unimib.it"),
    person("Alex","Graudenzi",role=c("aut"),email="alex.graudenzi@unimib.it"),
    person("Luca", "De Sano", role=c("aut"), email="luca.desano@gmail.com",
                comment = c(ORCID = "0000-0002-9618-3774")),
    person("Gianluca","Ascolani",role=c("aut"),email="gianluca.ascolani@unimib.it"))
Depends: R (>= 4.2.0)
Imports: curl, igraph, foreach, doParallel, sortable, dplyr, forcats,
        data.tree, graphics, grDevices, parallel, RColorBrewer, Rfast,
        stats, SummarizedExperiment, utils, purrr, stringi, stringr,
        Matrix, tidyr, jsonlite, readr, configr, DT, tools, fs,
        data.table, htmltools, htmlwidgets, bsplus, shinyvalidate,
        shiny, shinythemes, shinyFiles, shinyjs, shinyBS,
        shinydashboard, biomaRt, callr, logr, ggplot2, svglite
Suggests: BiocGenerics, BiocStyle, testthat, knitr, rmarkdown
Name: LACE: an R package for the inference of longitudinal cancer
        evolution models
Description: 
    LACE is an algorithmic framework that processes single-cell somatic mutation profiles from cancer samples collected at different time points and in distinct experimental settings, to produce longitudinal models of cancer evolution. The approach solves a Boolean Matrix Factorization problem with phylogenetic constraints, by maximizing a weighed likelihood function computed on multiple time points.
Encoding: UTF-8
License: file LICENSE
URL: https://github.com/BIMIB-DISCo/LACE
BugReports: https://github.com/BIMIB-DISCo/LACE
biocViews: BiomedicalInformatics, SingleCell, SomaticMutation
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/LACE
git_branch: RELEASE_3_23
git_last_commit: 5636e19
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:36:30 UTC; biocbuild
Author: Daniele Ramazzotti [aut] (ORCID:
    <https://orcid.org/0000-0002-6087-2666>),
  Fabrizio Angaroni [aut],
  Davide Maspero [cre, aut],
  Alex Graudenzi [aut],
  Luca De Sano [aut] (ORCID: <https://orcid.org/0000-0002-9618-3774>),
  Gianluca Ascolani [aut]
Maintainer: Davide Maspero <d.maspero@campus.unimib.it>
Built: R 4.6.0; ; 2026-04-29 04:48:09 UTC; unix
