# Generated by roxygen2: do not edit by hand

export(LACE)
export(LACEview)
export(compute.mutation.distance)
export(compute.variants.error.rates)
export(lacedata)
export(longitudinal.tree.plot)
import(DT)
import(RColorBrewer)
import(SummarizedExperiment, except = c("trim","shift", "show"))
import(biomaRt, except = c("show","select"))

import(bsplus)
import(callr)
import(configr)
import(curl, except = c("parse_date"))
import(data.table, except = c("first","last", "between"))
import(data.tree)
import(doParallel)
import(dplyr)
import(forcats)
import(foreach)
import(fs)
import(ggplot2)
import(grDevices)
import(graphics, except = c("box"))
import(htmltools)
import(jsonlite)
import(logr)
import(parallel)
import(readr)
import(shiny, except = c("runExample", "dataTableOutput", "renderDataTable","validate"))
import(shinyBS)
import(shinyFiles)
import(shinydashboard)
import(shinyjs)
import(shinythemes)
import(shinyvalidate)
import(sortable)
import(stringi)
import(stringr)
import(svglite)
import(tidyr)
import(tools)
import(utils)
importFrom(Matrix,rankMatrix)
importFrom(Rfast,rowMaxs)
importFrom(grDevices,colorRampPalette)
importFrom(jsonlite,toJSON)
importFrom(purrr,detect_index)
importFrom(purrr,is_null)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(tidyr,gather)
