Lace - Syntax of Allow and Deny statements
==========================================

These access control statements start `allow` or `deny`, followed by a single
token reason to be returned to the caller (thus the reason should be quoted if
it contains spaces), followed by zero or more definition names, all of which
must pass for the statement to execute.

For example:

    allow "Administrators can do anything" is-admin
    deny "Plebs may do nothing" is-pleb
    
    allow ''

As with definitions, includes, etc, if rule names are prefixed with an
exclamation point then their sense is inverted, e.g.

    deny "Only admins may alter hooks" altering-hooks !is-admin

